/**
 * @#D Azarewicz:2.18#@##1## 05 Jan 2017              DAZAR1    ::::::@@TestLog.Cmd (c) David Azarewicz 2017
 * V1.7 16-Sep-2011 changed APM.SYS to APM.ADD for ACPI log
 * V1.8 31-Dec-2011 Added current directory to search path
 * V1.9 02-Jan-2012 Added dsl file to log, fixed some errors
 * V2.0 29-Jan-2012 Added test for debug PSD in acpi log
 * V2.1 29-Feb-2012 Added Multimac r8110 logging
 * V2.2 20-Mar-2012 Added Multimac r8169, e1000e, nveth logging, removed r8110
 * V2.3 07-Sep-2012 Fixed search for PCI.EXE
 * V2.4 11-Nov-2012 Added support for Panorama
 * V2.5 16-Jul-2013 Added support for USB
 * V2.6 29-Jul-2013 Added support for AHCI
 * V2.7 01-Nov-2013 Added lantran.log to Multimac logs
 * V2.8 12-Jan-2013 Added SysInfo functions, usb device dump
 * V2.9 21-Aug-2014 Enhancements to ACPI logs
 * V2.10 21-Aug-2014 Enhancements to USB logs
 * V2.11 21-Feb-2015 Added drvutil support for USB
 * V2.12 27-Mar-2015 Added psfiles data for USB
 * V2.13 01-Sep-2015 Added support for "MM" drivers
 * V2.14 01-Mar-2016 Added support for JFS
 * V2.15 01-Sep-2016 Added support for log from the retail PSD, fixed PCI.EXE search to use OSDIR.
 * V2.16 01-Nov-2016 Added set/endlocal, tests for unsupported configurations
 * V2.17 23-Nov-2016 Added generic log feature
 * V2.18 05-Jan-2017 Added more features
 * Written by and Copyright (c) 2010-2017 David Azarewicz http://88watts.net
 */
TestLogVersion='2.18';
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

PARSE ARG TestId Arg2;

PARSE SOURCE Junk Junk CmdName;
CmdName=TRANSLATE(FILESPEC('name', CmdName));
select;
  when (CmdName='ACPILOG.CMD') then SysId='acpi';
  when (CmdName='UNIAUDLOG.CMD') then SysId='uniaud';
  when (CmdName='GENMACLOG.CMD') then SysId='genmac';
  when (CmdName='SANELOG.CMD') then SysId='sane';
  when (CmdName='PANORAMALOG.CMD') then SysId='panorama';
  when (CmdName='USBLOG.CMD') then SysId='usb';
  when (CmdName='AHCILOG.CMD') then SysId='ahci';
  otherwise do;
    SysId=TestId;
    TestId=Arg2;
  end;
end;

SysId=TRANSLATE(SysId);
CurDir=Directory();
rc=setlocal();

ENV='OS2ENVIRONMENT';
Host=VALUE('HOSTNAME',,ENV);
if (Host = '') then Host = "Unknown";
MyDate=DATE('S');
HostDate=Host||'-'||MyDate;
HeaderLine='File created '||DATE('N')||' '||TIME('N')||' by testlog.cmd v'||TestLogVersion||' available at http://88watts.net';
BootDrive=SysBootDrive();
TmpDirSlash = STRIP(value('TMP',,ENV),'T','\') || '\';
EtcDir = STRIP(value('ETC',,ENV),'T','\');
LogFilesDir = STRIP(value('LOGFILES',,ENV),'T','\');
if (LogFilesDir = '') then LogFilesDir=BootDrive||'\var\log';
rc=VALUE('PATH', CurDir||';'||VALUE('PATH',,ENV), ENV);
/* find PCI.EXE */
PCIEXE=SysSearchPath('PATH', 'PCI.EXE');
if (PCIEXE='') then do
  OsDir=VALUE('OSDIR',,ENV);
  call SysFileTree OsDir||'\pci.exe', 'file', 'FSO';
  if (file.0=0) then do
    say "PCI.EXE not found";
    signal FAILEXIT;
  end
  PCIEXE=file.1;
end
DRVUTILEXE=SysSearchPath('PATH', 'DRVUTIL.EXE');
ACPISTATEXE=SysSearchPath('PATH', 'AcpiStat.EXE');
NeedShortFileNames=(BootDrive='Z:');

'@echo off'
do while (QUEUED() \= 0); PULL; end;

select
  when (SysId="UNIAUD") then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    MMBase = STRIP(value('MMBASE',,'OS2ENVIRONMENT'),'T',';');
    PkgVer = PkgVersion(MMBase||'\uniaud32.sys');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'uniaud.log';
    else LogFile = TmpDirSlash || HostDate || '-uniaud-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||MMBase||'\uniaud32.sys');
    rc = DoCommand(LogFile, 'bldlevel '||MMBase||'\uniaud16.sys');
    rc = DoCommand(LogFile, 'unimix -card');
    rc = DoCommand(LogFile, 'unimix -list');
    rc = DoCommand(LogFile, 'unimix -pcms');
    rc = DoCommand(LogFile, 'unimix -powerget');
    rc = DoCommand(LogFile, 'type alsahlp$');
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  when (SysId = "GENMAC") then do
    PARSE VALUE SysIni( , 'Generic MAC Wrapper Driver', 'Path') WITH GMBase'0'x;
    IF ((GMBase = '') | (GMBase = 'ERROR:')) THEN DO
       SAY 'error: cannot find GenMac Installation.';
       signal FAILEXIT;
    END
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(GMBase||'\driver\genm32w.os2');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then do
      LogFile = TmpDirSlash||'genmac.log';
      ZipFile = TmpDirSlash||'genmac.zip';
    end
    else do
      LogFile = TmpDirSlash || HostDate || '-genmac-' || PkgVer || '.log';
      ZipFile = TmpDirSlash || HostDate || '-genmac-' || PkgVer || '.zip';
    end
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||GMBase||'\driver\genm32w.os2');
    rc = DoCommand(LogFile, 'type wrnddb$');
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    'zip -j -q 'ZipFile' 'LogFile
    say ZipFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  when (SysId="SANE") then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    SaneBase = 'e:\programs\tame';
    PkgVer = 'V';
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'sane.log';
    else LogFile = TmpDirSlash || HostDate || '-sane-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    'set SANE_DEBUG_microtek2=255';
    'set SANE_DEBUG_SANEI_USB=255';
    'set SANE_DEBUG_DLL=255';
    rc = DoCommand(LogFile, 'scanimage -L');
    'set SANE_DEBUG_microtek2=';
    'set SANE_DEBUG_SANEI_USB=';
    'set SANE_DEBUG_DLL=';
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  when (SysId = "ACPI") then do
    address CMD 'acpistat CheckDebugPSD >NUL 2>&1';
    if (rc > 0) then do;
      say "The debug version of the PSD is not running.";
      say "Only limited information can be gathered without the debug PSD.";
    end;
    TmpStr = FileFromAcpiDaemonCfg('AcpiLog', 1);
    if (TmpStr \= '') then do
      say "The following line is in your acpid.cfg file: "||TmpStr;
      say "Please remove or comment out this line, reboot, and try again.";
      say "Press enter when ready...";
      pull Answer;
      signal FAILEXIT;
    end
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(BootDrive||'\os2\boot\acpi.psd');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then do
      LogFile = TmpDirSlash||'acpi.log';
      LogDir = '';
      ZipFile = '';
    end
    else do
      LogDir = TmpDirSlash || 'acpilog.tmp';
      LogFile = TmpDirSlash || HostDate || '-acpi-' || PkgVer || '.log';
      ZipFile = TmpDirSlash || HostDate || '-acpi-' || PkgVer || '.zip';
    end
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\acpi.psd');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\dll\acpi32.dll');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\apm.add');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\acpidaemon.exe');
    rc = DoCommand(LogFile, 'type acpica$');
    rc=AddFileToLog(LogFile, EtcDir||'\acpid.cfg');
    TmpStr = FileFromAcpiDaemonCfg('LogFile', 0);
    if (TmpStr\='') then rc=AddFileToLog(LogFile, TmpStr);
    rc = AddSysInfoToLogEnd(LogFile);
    if (ZipFile\='') then do
      rc=SysMkDir(LogDir);
      call SysFileTree LogDir||'\*', 'file', 'FO'
      do i=1 to file.0
        call SysFileDelete file.i
      end
      rc=Directory(LogDir);
      if (SysSearchPath('PATH', 'acpidump.exe')='') then do
        rc = DoCommand(LogFile, 'iasl -g');
      end
      else do
        rc = DoCommand(LogFile, 'acpidump -s');
        address CMD 'acpidump -b >nul 2>&1';
        address CMD 'iasl -d facp.dat >nul 2>&1';
        address CMD 'iasl -d dsdt.dat >nul 2>&1';
      end
      rc=Directory(CurDir);
      call SysFileTree LogDir||'\*.dsl', 'file', 'FO'
      do i=1 to file.0
        rc=AddFileToLog(LogFile, file.i);
      end
      address CMD 'copy '||LogFile||' '||LogDir||' >nul 2>&1';
      address CMD 'zip -j -q -X '||ZipFile||' '||LogDir||'\*';
      call SysFileTree LogDir||'\*', 'file', 'FO'
      do i=1 to file.0
        call SysFileDelete file.i
      end
      rc=SysRmDir(LogDir);
      say ZipFile||" has been created.";
    end
    say LogFile||" has been created.";
    if (ZipFile\='') then do
      do while (QUEUED() \= 0); PULL; end;
      say "Please attach the above ZIP file to your ticket. (See the ACPI readme).";
      say "Press enter when ready...";
      pull Answer;
    end
  end
  when (SysId = "PANORAMA") then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(BootDrive||'\os2\dll\vbe2grad.dll');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'panorama.log';
    else LogFile = TmpDirSlash || HostDate || '-panorama-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\dll\vbe2grad.dll');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\dll\panogrex.dll');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\gradd.sys');
    rc = DoCommand(LogFile, 'PanoUtil -q -i -v -s');
    rc = AddFileToLog(LogFile, LogFilesDir||'\vbe2grad.log');
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  when (SysId = "USB") then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(BootDrive||'\os2\boot\usbehcd.sys');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'usb.log';
    else LogFile = TmpDirSlash || HostDate || '-usb-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbehcd.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbohcd.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbuhcd.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbd.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbhid.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbcom.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbkbd.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbprt.sys');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\usbmsd.add');
    rc = DoCommand(LogFile, 'bldlevel '||FileFromConfigSys('usbmouse.sys'));
    TmpStr=FileFromConfigSys('amouse.sys');
    if (TmpStr \= '') then rc = DoCommand(LogFile, 'bldlevel '||TmpStr);
    TmpStr=FileFromConfigSys('mouse.sys');
    if (TmpStr \= '') then rc = DoCommand(LogFile, 'bldlevel '||TmpStr);
    TmpStr=FileFromConfigSys('usbresmg.sys');
    if (TmpStr \= '') then rc = DoCommand(LogFile, 'bldlevel '||TmpStr);
    TmpStr=FileFromLibpath('USBCALLS.DLL');
    if (TmpStr \= '') then rc = DoCommand(LogFile, 'bldlevel '||TmpStr);
    if (DRVUTILEXE\='') then rc=DoCommand(LogFile, 'drvutil -t -u');
    rc = DoCommand(LogFile, 'type usbd$');
    rc = DoCommand(LogFile, 'type usbhid$');
    rc = DoCommand(LogFile, 'type usbkbd$');
    rc = DoCommand(LogFile, 'type usbmou$');
    rc = DoCommand(LogFile, 'type usbehcd$');
    rc = DoCommand(LogFile, 'type usbehc2$');
    rc = DoCommand(LogFile, 'type usbehc3$');
    rc = DoCommand(LogFile, 'type usbehc4$');
    rc = DoCommand(LogFile, 'type usbehc5$');
    rc = DoCommand(LogFile, 'type usbehc6$');
    rc = DoCommand(LogFile, 'type usbohcd$');
    rc = DoCommand(LogFile, 'type usbohc2$');
    rc = DoCommand(LogFile, 'type usbohc3$');
    rc = DoCommand(LogFile, 'type usbohc4$');
    rc = DoCommand(LogFile, 'type usbohc5$');
    rc = DoCommand(LogFile, 'type usbohc6$');
    rc = DoCommand(LogFile, 'type usbuhcd$');
    rc = DoCommand(LogFile, 'type usbuhc2$');
    rc = DoCommand(LogFile, 'type usbuhc3$');
    rc = DoCommand(LogFile, 'type usbuhc4$');
    rc = DoCommand(LogFile, 'type usbuhc5$');
    rc = DoCommand(LogFile, 'type usbuhc6$');
    rc = DoCommand(LogFile, 'psfiles | find "USB"');
    rc = AddUsbDumpToLog(LogFile);
    if (rc) then do
      say "WARNING! Incorrect USBCALLS.DLL/USBRESMG.SYS installed.";
      say "         Please reinstall and try again.";
    end
    else do
      rc = AddSysInfoToLogEnd(LogFile);
      say LogFile||" has been created.";
      say "Please attach the above log file to your ticket. Do not zip the log file.";
    end
  end
  when (SysId = "AHCI") | (SysId="DISK") then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(BootDrive||'\os2\boot\os2ahci.add');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'ahci.log';
    else LogFile = TmpDirSlash || HostDate || '-ahci-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\os2ahci.add');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\danis506.add');
    rc = DoCommand(LogFile, 'type os2ahci$');
    rc = DoCommand(LogFile, 'type ibms506$');
    if (DRVUTILEXE\='') then rc=DoCommand(LogFile, 'drvutil -a');
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  when (SysId = "JFS") then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(BootDrive||'\os2\jfs.ifs');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'jfs.log';
    else LogFile = TmpDirSlash || HostDate || '-jfs-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\jfs.ifs');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\dll\ujfs.dll');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\install\jfs.ifs');
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\install\ujfs.dll');
    rc = DoCommand(LogFile, 'cachejfs');
    rc = DoCommand(LogFile, 'cstats');
    JFSSTATS=SysSearchPath('PATH', 'jfsstats.exe');
    if (JFSSTATS\='') then rc = DoCommand(LogFile, 'jfsstats');
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  when (SysId="R8169") | (LEFT(SysId,5)="E1000") | (SysId="NVETH") | (LEFT(SysId,2)="MM")then do
    say "Please type a short description of the test result (1 line):";
    PARSE PULL TestResult;
    PkgVer = PkgVersion(BootDrive||'\ibmcom\macs\'||SysId||'.os2');
    if (TestId \= "") then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||SysId||'.log';
    else LogFile = TmpDirSlash || HostDate || '-'||SysId||'-' || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\ibmcom\macs\'||SysId||'.os2');
    rc = DoCommand(LogFile, 'type '||SysId||'$');
    rc=AddFileToLog(LogFile, BootDrive||'\ibmcom\lantran.log');
    rc=AddFileToLog(LogFile, BootDrive||'\ibmcom\protocol.ini');
    if (DRVUTILEXE\='') then rc=DoCommand(LogFile, 'drvutil -m '||SysId);
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
  otherwise do
    say 'testlog.cmd v'||TestLogVersion||' available at http://88watts.net';
    say
    say 'Performing a generic system log file.';
    say "To make a specific log file - Usage: testlog <driver> [testid]";
    say "  where <driver> can be one of the following:";
    say "  uniaud genmac sane acpi nveth panorama usb ahci disk jfs e1000b mm*";
    say
    say "Please type a short description of the problem (1 line):";
    PARSE PULL TestResult;
    PkgVer='';
    if (TestId \= '') then PkgVer = PkgVer || "-" || TestId;
    if (NeedShortFileNames) then LogFile = TmpDirSlash||'testlog.log';
    else LogFile = TmpDirSlash || HostDate || PkgVer || '.log';
    rc=SysFileDelete(LogFile);
    rc=LINEOUT(LogFile, HeaderLine);
    rc=LINEOUT(LogFile, "Test result: " || TestResult);
    rc=LINEOUT(LogFile);
    rc = AddSysInfoToLogBeg(LogFile);
    rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2\boot\acpi.psd');
    rc = AddSysInfoToLogEnd(LogFile);
    say LogFile||" has been created.";
    say "Please attach the above log file to your ticket. Do not zip the log file.";
  end
end

FAILEXIT:
rc=Directory(CurDir);
rc=endlocal();
exit;

CheckConfiguration: procedure expose ACPISTATEXE;
  parse arg LogFile;

  RetVal = 0;
  ENV='OS2ENVIRONMENT';
  MyMsg="Unsupported configuration detected:";
  Line = LEFT('----- Configuration Check ', 80, '-');
  rc=LINEOUT(LogFile, Line);
  if (ACPISTATEXE\='') then do
    address CMD ACPISTATEXE||' CheckConfiguration >NUL 2>&1';
    if (rc > 0) then do
      Bits = X2B(C2X(rc));
      BitLen = Length(Bits);
      Msg='';
      if (SubStr(Bits, BitLen, 1)=1) then Msg=Msg||" USB";
      if (SubStr(Bits, BitLen-1, 1)=1) then Msg=Msg||" QSINIT";
      if (SubStr(Bits, BitLen-2, 1)=1) then Msg=Msg||" MCHNCHK";
      rc=LineOut(LogFile, MyMsg||Msg);
      RetVal = 1;
    end
  end
  if (Translate(Right(Value('OS2_SHELL',,ENV),7))\='CMD.EXE') then rc=LineOut(LogFile, MyMsg||' OS2_SHELL');
  rc=LineOut(LogFile);
  return RetVal;

FileFromConfigSys: procedure expose BootDrive;
  parse upper arg SearchString;
  ConfigSys=BootDrive||'\config.sys';
  FileName='';
  do while LINES(ConfigSys) \= 0;
    Line=STRIP(TRANSLATE(LINEIN(ConfigSys)));
    if (LEFT(Line,3)='REM') then iterate;
    if (POS(SearchString, Line) = 0) then iterate;
    FileName = WORD(STRIP(SUBSTR(Line, POS('=',Line)+1)),1);
    TmpStr=FILESPEC('N',FileName);
    if (TmpStr=SearchString) then leave;
    FileName='';
  end;
  rc=stream(configSys,'c','close');
  /* say 'FileFromConfigSys: Search='||SearchString||' Found="'||FileName||'"'; */
  return FileName;

FileFromLibpath: procedure expose BootDrive ENV;
  parse upper arg SearchString;
  ConfigSys=BootDrive||'\config.sys';
  FileName='';
  rc=SysFileSearch('LIBPATH', ConfigSys, 'libp.');
  if (rc = 0) then rc = (libp.0 = 0);
  if (rc \= 0) then return FileName;
  lix = libp.0; /* Point to last occurrence */
  DO FOREVER
    PARSE UPPER VALUE libp.lix WITH Arg1 '=' Arg2;
    if (Arg1='LIBPATH') then leave;
    lix = lix-1;
    IF (lix = 0) THEN return FileName;
  END
  rc=Value('MyTmp', Arg2, ENV);
  FileName=SysSearchPath('MyTmp', 'USBCALLS.DLL');
  rc=Value('MyTmp', '', ENV);
  return FileName;

DoCommand: procedure;
  parse arg LogFile, Cmd;
  Line = LEFT('----- Output of: '||Cmd||' ', 80, '-');
  rc=LINEOUT(LogFile, Line); rc=LINEOUT(LogFile);
  address CMD Cmd || ' 1>>'||LogFile||' 2>&1';
  RETURN(0);

PkgVersion: procedure;
  File = ARG(1);
  '@bldlevel 'File' 2>&1 | rxqueue'
  do while (QUEUED() > 0)
    PARSE PULL Line1':'Line2
    if (Line1 = "Revision") then leave;
    if (Line1 = "File Version") then leave;
    Line2 = "";
  end
  do while (QUEUED() \= 0); PULL; end;
  Return(STRIP(Line2,'B'));

CheckDebugVersion: procedure;
  File = ARG(1);
  address CMD '@bldlevel '||File||' 2>&1 | rxqueue'
  do while (QUEUED() > 0)
    PARSE PULL Line1':'Line2
    if (Line1 = "Signature") then leave;
    Line2 = "";
  end
  do while (QUEUED() \= 0); PULL; end;
  Return(POS("Debug", Line2));

FileFromAcpiDaemonCfg: procedure expose EtcDir;
  parse upper arg SearchString, ReturnLine;
  DaemonCfg=EtcDir||'\acpid.cfg';
  Line='';
  do while LINES(DaemonCfg) \= 0;
    TmpLine=STRIP(LINEIN(DaemonCfg));
    if (LEFT(TmpLine,1)=';') then iterate;
    if (LEFT(TmpLine,1)='#') then iterate;
    if (LEFT(TmpLine,1)='%') then iterate;
    if (POS(SearchString, TRANSLATE(TmpLine)) > 0) then do
      Line=TmpLine;
      leave;
    end
  end;
  rc=stream(DaemonCfg,'c','close');
  if (ReturnLine) then return Line;
  if (Line='') then return Line;
  FileName = WORD(STRIP(SUBSTR(Line, POS('=',Line)+1)),1);
  return FileName;

AddFileToLog: procedure;
  parse arg LogFile, FileName;

  Line = LEFT('----- Contents of file: '||FileName||' ', 80, '-');
  rc=LINEOUT(LogFile, Line);
  do while LINES(FileName) \= 0;
    rc=LINEOUT(LogFile, LINEIN(FileName));
  end;
  rc=LINEOUT(LogFile);
  rc=stream(FileName,'c','close');
  return 0;

AddSysInfoToLogBeg: procedure expose BootDrive ACPISTATEXE;
  parse arg LogFile;
  rc = DoCommand(LogFile, 'dir /A '||BootDrive||'\os2*');
  rc = DoCommand(LogFile, 'bldlevel '||BootDrive||'\os2krnl');
  MYEXE=SysSearchPath('PATH', 'cpuspeed.exe');
  if (MYCMD\='') then rc = DoCommand(LogFile, 'cpuspeed');
  MyFile = stream(BootDrive||'\CONFIG.LDR','c','query exists');
  if (MyFile\='') then rc = AddFileToLog(LogFile, MyFile);
  rc=CheckConfiguration(LogFile);
  return 0;

AddSysInfoToLogEnd: procedure expose BootDrive PCIEXE;
  parse arg LogFile;
  rc = DoCommand(LogFile, PCIEXE||' -N');
  rc = DoCommand(LogFile, 'rmview -irq');
  rc = AddFileToLog(LogFile, BootDrive||'\CONFIG.SYS');
  rc = AddFileToLog(LogFile, BootDrive||'\AUTOEXEC.BAT');
  return 0;

AddUsbDumpToLog: procedure;
  parse arg LogFile;

  Line = LEFT('----- USB Device List ', 80, '-');
  rc=LINEOUT(LogFile, Line);

  signal on syntax name UsbError;
  rc=RxFuncAdd('UsbLoadFuncs', 'USBCALLS', 'USBLOADFUNCS');
  rc=UsbLoadFuncs();
  signal off syntax;

  signal on syntax name UsbContinue;
  drop DllMajor DllMinor UsbMajor UsbMinor;
  rc=RxUsbQueryVersion(DllMajor,DllMinor,UsbMajor,UsbMinor);
  if (rc=0) then do
    rc=LINEOUT(LogFile, 'Versions: DLL='||DllMajor||'.'||DllMinor||' USB='||UsbMajor||'.'||UsbMinor);
    if (DllMajor<11) then signal UsbError;
    if (UsbMajor<11) then signal UsbError;
    if (DllMinor<11) then signal UsbError;
    if (UsbMinor<11) then signal UsbError;
  end
UsbContinue:
  signal off syntax;

  drop NumDevices;
  rc=RxUsbQueryNumberDevices(NumDevices);
  if (rc=0) then rc=LINEOUT(LogFile, NumDevices||' USB devices attached to the system');
  else do
    rc=LINEOUT(LogFile, 'Error quering number of devices');
    rc=LINEOUT(LogFile);
    return 0;
  end

  do d=1 to NumDevices
    rc=LINEOUT(LogFile, "Device "||d);
    drop Report;
    rc=RxUsbQueryDeviceReport(d, Report);
    if (rc\=0) then do
      rc=LINEOUT(LogFile, "  Query device failed. rc="||rc);
      iterate;
    end
    ReportLen=LENGTH(Report);
    Size=X2D(SUBSTR(Report,1,2));
    Type=X2D(SUBSTR(Report,3,2));
    if (Size\=18) | (Type\=1) then do
      rc=LINEOUT(LogFile, "  Bad descriptor Len="||Size||" Type="||Type);
      iterate;
    end
    DeviceClass=SUBSTR(Report,9,2);
    DeviceSubClass=SUBSTR(Report,11,2);
    DeviceProtocol=SUBSTR(Report,13,2);
    MaxPacketSize=SUBSTR(Report,15,2);
    Vendor=SUBSTR(Report,19,2)||SUBSTR(Report,17,2);
    Product=SUBSTR(Report,23,2)||SUBSTR(Report,21,2);
    NumConfigs=X2D(SUBSTR(Report,35,2));
    ManuString=X2D(SUBSTR(Report,29,2));
    ProductString=X2D(SUBSTR(Report,31,2));
    SerialString=X2D(SUBSTR(Report,33,2));
    rc=LINEOUT(LogFile, "  Class="||DeviceClass||" SubClass="||DeviceSubClass||" Protocol="||DeviceProtocol);
    rc=LINEOUT(LogFile, "  NumConfigs="||NumConfigs||" MaxPacketSize="||MaxPacketSize);
    rc=LINEOUT(LogFile, "  ID="||Vendor||":"||Product);
    rc=LINEOUT(LogFile, "    ManuString="||ManuString);
    rc=LINEOUT(LogFile, "    ProductString="||ProductString);
    rc=LINEOUT(LogFile, "    SerialString="||SerialString);
    NumItems=NumConfigs;
    ConfigIX=0;
    InterfaceIX=0;
    EndpointIX=0;
    Base=Size*2+1;
    do while (NumItems>0) & (Base < ReportLen)
      Size=X2D(SUBSTR(Report,Base+0,2));
      Type=X2D(SUBSTR(Report,Base+2,2));
      select
        when (Size=0) & (Type=0) then leave;
        when (Size=9) & (Type=2) then do /* configuration */
          ConfigIX=ConfigIX+1;
          NumItems=NumItems-1;
          rc=LINEOUT(LogFile, "  Configuration "||ConfigIX);
          Len=SUBSTR(Report,Base+6,2)||SUBSTR(Report,Base+4,2);
          NumInterfaces=X2D(SUBSTR(Report,Base+8,2));
          NumItems=NumItems+NumInterfaces;
          ConfigString=X2D(SUBSTR(Report,Base+12,2));
          rc=LINEOUT(LogFile, "    NumInterfaces="||NumInterfaces);
          rc=LINEOUT(LogFile, "    ConfigString="||ConfigString);
          InterfaceIX=0;
        end
        when (Size=9) & (Type=4) then do /* Interface */
          InterfaceIX=InterfaceIX+1;
          NumItems=NumItems-1;
          rc=LINEOUT(LogFile, "    Interface "||InterfaceIX);
          NumEndpoints=X2D(SUBSTR(Report,Base+8,2));
          NumItems=NumItems+NumEndpoints;
          InterfaceClass=X2D(SUBSTR(Report,Base+10,2));
          InterfaceSubClass=X2D(SUBSTR(Report,Base+12,2));
          InterfaceProtocol=X2D(SUBSTR(Report,Base+14,2));
          InterfaceString=X2D(SUBSTR(Report,Base+16,2));
          String='';
          if (InterfaceClass=1) then String=' Audio';
          if (InterfaceClass=2) then String=' Communication';
          if (InterfaceClass=3) then String=' HID';
          if (InterfaceClass=6) then String=' Image';
          if (InterfaceClass=7) then String=' Printer';
          if (InterfaceClass=8) then String=' MSD';
          if (InterfaceClass=14) then String=' Video';
          if (InterfaceClass=224) then String=' Wireless Controller';
          if (InterfaceClass=255) then String=' Vendor Specific';
          rc=LINEOUT(LogFile, "      InterfaceClass="||InterfaceClass||String);
          rc=LINEOUT(LogFile, "      InterfaceSubClass="||InterfaceSubClass);
          String='';
          if (InterfaceClass=3) then do
            if (InterfaceProtocol=1) then String=' Keyboard';
            if (InterfaceProtocol=2) then String=' Mouse';
          end
          if (InterfaceClass=224) then do
            if (InterfaceSubClass=1) then do
              if (InterfaceProtocol=1) then String=' Bluetooth';
              if (InterfaceProtocol=2) then String=' UWB Radio';
              if (InterfaceProtocol=3) then String=' Remote NDIS';
              if (InterfaceProtocol=4) then String=' Bluetooth AMP';
            end
          end
          rc=LINEOUT(LogFile, "      InterfaceProtocol="||InterfaceProtocol||String);
          rc=LINEOUT(LogFile, "      InterfaceString="||InterfaceString);
          rc=LINEOUT(LogFile, "      NumEndpoints="||NumEndpoints);
          EndpointIX=0;
        end
        when (Size=7) & (Type=5) then do /* Endpoint */
          EndpointIX=EndpointIX+1;
          NumItems=NumItems-1;
          rc=LINEOUT(LogFile, "      Endpoint "||EndpointIX);
          EndpointAddress=SUBSTR(Report,Base+4,2);
          Attributes=SUBSTR(Report,Base+6,2);
          MaxLen=SUBSTR(Report,Base+10,2)||SUBSTR(Report,Base+8,2);
          rc=LINEOUT(LogFile, "        Address=0x"||EndpointAddress);
          rc=LINEOUT(LogFile, "        Attributes=0x"||Attributes);
          rc=LINEOUT(LogFile, "        MaxPacketSize=0x"||MaxLen);
        end
        when (Size=9) & (Type=33) then do /* HID descriptor */
          rc=LINEOUT(LogFile, "      HID Descriptor Release Number=0x"||SUBSTR(Report,Base+6,2)||SUBSTR(Report,Base+4,2));
          rc=LINEOUT(LogFile, "        Country=0x"||SUBSTR(Report,Base+8,2));
          rc=LINEOUT(LogFile, "        NumClassDescriptors=0x"||SUBSTR(Report,Base+10,2));
          rc=LINEOUT(LogFile, "        ReportDescriptorType=0x"||SUBSTR(Report,Base+12,2));
          rc=LINEOUT(LogFile, "        TotalDescriptorLength=0x"||SUBSTR(Report,Base+16,2)||SUBSTR(Report,Base+14,2));
        end
        otherwise do
          rc=LINEOUT(LogFile, "      Descriptor: Type="||Type||" Size="||Size);
          TmpStr='       ';
          do i=0 to (Size-1)*2 by 2
            TmpStr=TmpStr||' '||SUBSTR(Report,Base+i,2);
          end
          rc=LINEOUT(LogFile, TmpStr);
        end
      end
      Base=Base+Size*2;
    end
  end
  rc=LINEOUT(LogFile);
  return 0;

UsbError:
  rc=LINEOUT(LogFile);
  return 1;

